/* GLM computations based on the RFX GLM ANOVA exercise data
 
03-04-19: changed 'BrainVoyagerQX' object into 'BrainVoyager' for BV21
15-02-24: updated to BV23 
*/

// choose from these constants
const FILETYPE_FMR = 1;
const FILETYPE_VTC = 2;
const NORMALISE_TIMECOURSE_PSC = 1;
const NORMALISE_TIMECOURSE_ZTRF = 2;
const NORMALIZE_TIMECOURSE_ZTRF_BASE = 3;
const GLMTYPE_SINGLESTUDY = 1;
const GLMTYPE_MULTISTUDY = 2;
const GLMTYPE_RANDOMEFFECTS = 3;
const PREDICTORS_SEPARATESTUDY = 1;
const PREDICTORS_SEPARATESUBJECTS = 2;

// select here the parameters for your GLM
var glmtype                 = GLMTYPE_SINGLESTUDY;
var correctSerialCorr       = true;
var normaliseTimeCourse     = NORMALISE_TIMECOURSE_PSC; // choose from constants above
var predSeparation          = PREDICTORS_SEPARATESTUDY;
var filetype                = FILETYPE_VTC;
var mainpath                = BrainVoyager.BrowseDirectory("Please select the directory with the data") + "/";//"/Users/BVuser/Data/rfx/";

// for VTC-GLMs: write here the name of the anatomical file
var anatname = mainpath + "Average_Tal.vmr";
var mdmname = mainpath + "RFX_TAL.mdm";

// write here the design matrix names
var desmat1 = mainpath + "run1_SCSAI2_3DMCTS_LTR_THPFFT3c_TAL.sdm";
//var desmat2 = mainpath + "run2_SCSAI2_3DMCTS_LTR_THPFFT3c_TAL.sdm";

// write here the filenames
var nroffiles = 10;
var filenames = new Array(nroffiles);
filenames[0] = mainpath + "Sub1_run1_SCSAI2_3DMCTS_LTR_THPFFT3c_TAL.vtc";
filenames[1] = mainpath + "Sub2_run1_SCSAI2_3DMCTS_LTR_THPFFT3c_TAL.vtc";
filenames[2] = mainpath + "Sub3_run1_SCSAI2_3DMCTS_LTR_THPFFT3c_TAL.vtc";
filenames[3] = mainpath + "Sub4_run1_SCSAI2_3DMCTS_LTR_THPFFT3c_TAL.vtc";
filenames[4] = mainpath + "Sub5_run1_SCSAI2_3DMCTS_LTR_THPFFT3c_TAL.vtc";
filenames[5] = mainpath + "Sub6_run1_SCSAI2_3DMCTS_LTR_THPFFT3c_TAL.vtc";
filenames[6] = mainpath + "Sub7_run1_SCSAI2_3DMCTS_LTR_THPFFT3c_TAL.vtc";
filenames[7] = mainpath + "Sub8_run1_SCSAI2_3DMCTS_LTR_THPFFT3c_TAL.vtc";
filenames[8] = mainpath + "Sub9_run1_SCSAI2_3DMCTS_LTR_THPFFT3c_TAL.vtc";
filenames[9] = mainpath + "Sub10_run1_SCSAI2_3DMCTS_LTR_THPFFT3c_TAL.vtc";
var desmatfile;

// Generate_MDM_file(mdmname); // In BrainVoyager QX 2.1 only for VTCs
Compute_GLMs();
// RunRandomEffectsGLM();

// ~~ don't change from here ~~

function Compute_GLMs() {
    
    var doc, funcfile, desmatfile;
    if (glmtype == GLMTYPE_SINGLESTUDY) {
        BrainVoyager.PrintToLog("Start single-study GLMs...");
        desmatfile = desmat1;
        for (var counter = 0; counter < nroffiles; counter++) {     
                funcfile = filenames[counter];
           //     desmatfile = desmatfiles[counter];
            if (filetype == FILETYPE_FMR) {        
                BrainVoyager.PrintToLog("Opening " + funcfile + "..."); // functional data (*.fmr + *.stc)
                doc = BrainVoyager.OpenDocument(funcfile); 
            } else if (filetype == FILETYPE_VTC){
                BrainVoyager.PrintToLog("Opening " + anatname + "..."); // anatomical data (*.vmr)
                doc = BrainVoyager.OpenDocument(anatname); 
                BrainVoyager.PrintToLog("Linking " + funcfile + "..."); // functional data (*.vtc)              
                doc.LinkVTC(funcfile); 
            }
            doc.CorrectForSerialCorrelations = correctSerialCorr;
            BrainVoyager.PrintToLog("Loading " + desmatfile + "...");
            BrainVoyager.PrintToLog("Correct serial correlations: " + (doc.CorrectForSerialCorrelations ? "yes" : "no"));
            doc.LoadSingleStudyGLMDesignMatrix(desmatfile);    // design matrix (*.rtc, *.sdm)
            doc.ComputeSingleStudyGLM();
            
            var glmname = removeExtension(funcfile) + ".glm";        // bool     rename(const QString &newName)
            BrainVoyager.PrintToLog("Saving " + glmname + "...");    
            doc.SaveGLM(glmname); 
            doc.ShowGLM(); 
            doc.Close();
        } // end counter          
    
    } else if  (glmtype == GLMTYPE_MULTISTUDY) {
        
         BrainVoyager.PrintToLog("Opening " + anatname + "..."); // anatomical data (*.vmr)
         doc = BrainVoyager.OpenDocument(anatname); 
         doc.LoadMultiStudyGLMDefinitionFile(mdmname);
         doc.PSCTransformStudies             = (normaliseTimeCourse == NORMALISE_TIMECOURSE_PSC);
         doc.ZTransformStudies                 = (normaliseTimeCourse == NORMALISE_TIMECOURSE_ZTRF);
         doc.ZTransformStudiesBaselineOnly     = (normaliseTimeCourse == NORMALIZE_TIMECOURSE_ZTRF_BASE);
         doc.SeparationOfSubjectPredictors     = (predSeparation == PREDICTORS_SEPARATESUBJECTS);
         doc.SeparationOfStudyPredictors         = (predSeparation == PREDICTORS_SEPARATESTUDY);
         BrainVoyager.PrintToLog("Predictor separation: " + (doc.SeparationOfSubjectPredictors ? "per subject" : (doc.SeparationOfStudyPredictors ? "per run":"not set")));
         BrainVoyager.PrintToLog("Data normalization: " + (doc.ZTransformStudies ? "z-transform" : (doc.ZTransformStudiesBaselineOnly ? "z-transform (baseline)" : 
                                        (doc.PSCTransformStudies ? "percent signal change " : "not set"))));
         doc.ComputeMultiStudyGLM();
         var glmname = removeExtension(mdmname) + ".glm";        
         BrainVoyager.PrintToLog("Saving " + glmname + "...");    
         doc.SaveGLM(glmname);
         doc.ShowGLM();
    }
}

function Generate_MDM_file(mdmname) {

    BrainVoyager.ShowLogPane();
    BrainVoyager.PrintToLog("Start creating MDM...\nOpening anatomical file: " + anatname);
    var anat = BrainVoyager.OpenDocument(anatname);
    if (anat == undefined) {
        BrainVoyager.PrintToLog("Invalid anatomical filename.");
    } else {
        anat.ClearMultiStudyGLMDefinition();
        for (var counter = 0; counter < nroffiles; counter++) {
            BrainVoyager.PrintToLog("Adding:\n" + filenames[counter] + "\n" + desmatfile);// desmatfiles[counter]);
            anat.AddStudyAndDesignMatrix(filenames[counter], desmatfile); // desmatfiles[counter]);
        }
    //    var mdmname = removeExtension(anatname) + ".mdm";
        BrainVoyager.PrintToLog("Name for MDM file: " + mdmname);
        anat.SaveMultiStudyGLMDefinitionFile(mdmname);
    }
}
    
function RunRandomEffectsGLM() {
    
    BrainVoyager.PrintToLog("Start random effects GLM... ");
    var anat = BrainVoyager.OpenDocument(anatname);
    anat.ClearMultiStudyGLMDefinition();
    anat.LoadMultiStudyGLMDefinitionFile(mdmname);
     anat.PSCTransformStudies             = (normaliseTimeCourse == NORMALISE_TIMECOURSE_PSC);
    anat.ZTransformStudies                 = (normaliseTimeCourse == NORMALISE_TIMECOURSE_ZTRF);
    anat.ZTransformStudiesBaselineOnly     = (normaliseTimeCourse == NORMALIZE_TIMECOURSE_ZTRF_BASE);
    anat.SeparationOfSubjectPredictors     = (predSeparation == PREDICTORS_SEPARATESUBJECTS);
    anat.SeparationOfStudyPredictors     = (predSeparation == PREDICTORS_SEPARATESTUDY);
    BrainVoyager.PrintToLog("Predictor separation: " + (anat.SeparationOfSubjectPredictors ? "per subject" : (anat.SeparationOfStudyPredictors ? "per run":"not set")));
    BrainVoyager.PrintToLog("Data normalization: " + (anat.ZTransformStudies ? "z-transform" : (anat.ZTransformStudiesBaselineOnly ? "z-transform (baseline)" : 
                                        (anat.PSCTransformStudies ? "percent signal change " : "not set"))));
    anat.ComputeRFXGLM();
    var rfxglmname = removeExtension(mdmname) + "_rfx.glm";
    BrainVoyager.PrintToLog("Saving " + rfxglmname + "...");    
    anat.SaveGLM(rfxglmname);
    anat.ShowGLM();
}

function removeExtension(filename) {
    var pos = filename.lastIndexOf(".");
    return filename.substring(0,pos);
}
